// Definicije funkcija uz klasu RFile.

#include "rfile.h"

int RFile::otvori (const char *ime, long duz) { // OTVARANJE DATOTEKE.
  if (duz < 0) return G_DUZ;            // Neispravna duzina zapisa.
  if (bro_zap >= 0) close ();           // Datot. otvorena  zatvori.
  if (duz) {                            // Formiranje nove datoteke.
    fstream::open (ime, ios::in | ios::out |
                        ios::noreplace | ios::binary) ;
    if (! good ())                                      return G_OTV;
    if (!fstream::write((char *)&duz, sizeof(long)) ||
        !flush()                                      ) return G_PIS;
    duz_zap = duz;
    bro_zap = 0;
  } else {                              // Otvaranje stare datoteke.
    fstream::open (ime, ios::in | ios::out |
                        ios::nocreate | ios::binary);
    if (! good ())                                         return G_OTV;
    if (! fstream::read ((char *)&duz_zap, sizeof (long))) return G_CIT;
    if (! seekg (0l, ios::end))                            return G_POZ;
    long k = tellg (); if ((k - sizeof (long)) % duz_zap)  return G_VEL;
    bro_zap = (k - sizeof (long)) / duz_zap;
  }
  tek_zap = 0;
  tek_poz = sizeof (long);
  return OK;
}

int RFile::close () {                           // ZATVARANJE DATOTEKE.
  fstream::close ();
  bro_zap = -1;
  return OK;
}

int RFile::write (const void *niz, long zap) {  // PISANJE ZAPISA.
  if (bro_zap == -1) return G_ZAT;
  if (zap < 0)       return G_BRO;
  if (zap) {                            // Direktan pristup.
    tek_poz = (zap-1) * duz_zap + sizeof (long);
    tek_zap = zap;
  } else {                              // Sekvencijalni pristup.
    tek_poz += duz_zap;
    tek_zap++;
  }
  if (tek_zap > bro_zap) bro_zap = tek_zap;
  if (! seekp (tek_poz))                          return G_POZ;
  if (! fstream::write ((char *)niz, duz_zap) ||
      ! flush ()                                ) return G_PIS;
  return OK;
}

int RFile::read (void *niz, long zap) {         // CITANJE ZAPISA.
  if (bro_zap == -1) return G_ZAT;
  if (zap < 0)       return G_BRO;
  if (zap) {                            // Direktan pristup.
    tek_poz = (zap-1) * duz_zap + sizeof (long);
    tek_zap = zap;
  } else {                              // Sekvencijalni pristup.
    tek_poz += duz_zap;
    tek_zap++;
  }
  if (zap > bro_zap) return G_ZAP;
  if (! seekg (tek_poz))                      return G_POZ;
  if (! fstream::read ((char *)niz, duz_zap)) return G_PIS;
  return OK;
}
